@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: IW44Image.h,v 1.13 2007/05/19 03:07:34 leonb Exp $
-// $Name: release_3_5_23 $
 
 #ifndef IW44IMAGE_H_
 #define IW44IMAGE_H_
@@ -161,8 +158,7 @@
 // Lizardtech has split the corresponding cpp file into a decoder and an encoder.
 // Only superficial changes.  The meat is mine.
 
-    @version
-    #$Id: IW44Image.h,v 1.13 2007/05/19 03:07:34 leonb Exp $# */
+*/
 //@{
 
 
@@ -597,7 +593,7 @@ public:
   void   zero(int n);
   // sparse representation
 private:
-  short **(pdata[4]);
+  short **pdata[4];
 };
 
 //---------------------------------------------------------------
