@@ -66,6 +66,7 @@
 #include "GString.h"
 #include "DjVuDocEditor.h"
 #include "DjVuDumpHelper.h"
+#include "DjVuMessageLite.h"
 #include "BSByteStream.h"
 #include "DjVuText.h"
 #include "DjVuAnno.h"
@@ -460,7 +461,11 @@ print_c_string(const char *data, int length, ByteStream &out, bool eightbit)
           char buf[5];
           static const char *tr1 = "\"\\tnrbf";
           static const char *tr2 = "\"\\\t\n\r\b\f";
+#if HAVE_SNPRINTF
+          snprintf(buf,sizeof(buf),"\\%03o", (int)(((unsigned char*)data)[0]));
+#else
           sprintf(buf,"\\%03o", (int)(((unsigned char*)data)[0]));
+#endif
           { // extra nesting for windows
             for (int i=0; tr2[i]; i++)
               if (*(char*)data == tr2[i])
@@ -1473,11 +1478,9 @@ get_text(const GP<DjVuFile> &file)
   const GP<ByteStream> bs(file->get_text());
   if (bs) 
     {
-      long int i=0;
       const GP<IFFByteStream> iff(IFFByteStream::create(bs));
       while (iff->get_chunk(chkid))
         {
-          i++;
           if (chkid == GUTF8String("TXTa")) 
             {
               GP<DjVuTXT> txt = DjVuTXT::create();
@@ -1768,7 +1771,11 @@ output(const GP<DjVuFile> &f, const GP<ByteStream> &out,
       const GP<ByteStream> ant(ByteStream::create());
       const GP<ByteStream> txt(ByteStream::create());
       char pagenumber[16];
+#if HAVE_SNPRINTF
+      snprintf(pagenumber,sizeof(pagenumber)," # page %d", pageno);
+#else      
       sprintf(pagenumber," # page %d", pageno);
+#endif
       if (flag & 1) 
         { 
           const GP<ByteStream> anno(f->get_anno());
@@ -2021,11 +2028,11 @@ command_set_thumbnails(ParsingByteStream &pbs)
 {
   GUTF8String sizestr = pbs.get_token();
   if (! sizestr)
-    sizestr = "128";
+    sizestr = "192";
   if (! sizestr.is_int() )
     verror("expecting integer argument");
   int size = atoi(sizestr);
-  if (size<32 || size >256) 
+  if (size < 32 || size > 512) 
     verror("size should be between 32 and 256 (e.g. 128)");
   g().doc->generate_thumbnails(size, callback_thumbnails, NULL);
   modified = true;
@@ -2160,7 +2167,7 @@ command_help(void)
           " . set-meta [<metafile>]  -- copies <metafile> into the metadata annotation tag\n"
           " . set-txt [<txtfile>]    -- copies <txtfile> into the hidden text chunk\n"
           " . set-xmp [<xmpfile>]    -- copies <xmpfile> into the xmp metadata annotation tag\n" 
-          " _ set-outline [<bmfile>] -- sets outline (bootmarks)\n"
+          " _ set-outline [<bmfile>] -- sets outline (bookmarks)\n"
           " _ set-thumbnails [<sz>]  -- generates all thumbnails with given size\n"
           "   set-rotation [+-]<rot> -- sets page rotation\n"
           "   set-dpi <dpi>          -- sets page resolution\n"
@@ -2315,7 +2322,8 @@ execute()
       G_CATCH(ex)
         {
           vprint("Error (%s): %s",
-                 (const char*)ToNative(token), ex.get_cause());
+                 (const char*)ToNative(token), 
+                 (const char *)DjVuMessageLite::LookUpUTF8(ex.get_cause()));
           if (! verbose)
             G_RETHROW;
         }
